@@ -56,8 +56,8 @@ class CfgYaml:
     key_dotfile_dst = 'dst'
     key_dotfile_link = 'link'
     key_dotfile_actions = 'actions'
-    key_dotfile_link_children = 'link_children'
     key_dotfile_noempty = 'ignoreempty'
+    key_dotfile_template = 'template'
 
     # profile
     key_profile_dotfiles = 'dotfiles'
@@ -83,12 +83,17 @@ class CfgYaml:
     key_settings_noempty = Settings.key_ignoreempty
     key_settings_minversion = Settings.key_minversion
     key_imp_link = Settings.key_link_on_import
+    key_settings_template = Settings.key_template_dotfile_default
 
     # link values
     lnk_nolink = LinkTypes.NOLINK.name.lower()
     lnk_link = LinkTypes.LINK.name.lower()
     lnk_children = LinkTypes.LINK_CHILDREN.name.lower()
 
+    # checks
+    allowed_link_val = [lnk_nolink, lnk_link, lnk_children]
+    top_entries = [key_dotfiles, key_settings, key_profiles]
+
     def __init__(self, path, profile=None, addprofiles=[], debug=False):
         """
         config parser
@@ -128,6 +133,8 @@ class CfgYaml:
         self._yaml_dict = self._load_yaml(self._path)
         # live patch deprecated entries
         self._fix_deprecated(self._yaml_dict)
+        # validate content
+        self._validate(self._yaml_dict)
 
         ##################################################
         # parse the config and variables
@@ -235,15 +242,15 @@ class CfgYaml:
 
         # process profile ALL
         self._resolve_profile_all()
-        # patch dotfiles paths
-        self._template_dotfiles_paths()
+        # template dotfiles entries
+        self._template_dotfiles_entries()
 
         if self._debug:
             self._dbg('########### {} ###########'.format('final config'))
             self._debug_entries()
 
     ########################################################
-    # outside available methods
+    # public methods
     ########################################################
 
     def resolve_dotfile_src(self, src, templater=None):
@@ -589,14 +596,19 @@ class CfgYaml:
                 v[self.key_trans_r] = v[self.old_key_trans_r]
                 del v[self.old_key_trans_r]
                 new[k] = v
-            # apply link value
             if self.key_dotfile_link not in v:
+                # apply link value if undefined
                 val = self.settings[self.key_settings_link_dotfile_default]
                 v[self.key_dotfile_link] = val
             # apply noempty if undefined
             if self.key_dotfile_noempty not in v:
                 val = self.settings.get(self.key_settings_noempty, False)
                 v[self.key_dotfile_noempty] = val
+            # apply template if undefined
+            if self.key_dotfile_template not in v:
+                val = self.settings.get(self.key_settings_template, True)
+                v[self.key_dotfile_template] = val
+
         return new
 
     def _add_variables(self, new, shell=False, template=True, prio=False):
@@ -916,26 +928,27 @@ class CfgYaml:
 
     def _fix_deprecated_link_by_default(self, yamldict):
         """fix deprecated link_by_default"""
-        key = 'link_by_default'
+        old_key = 'link_by_default'
         newkey = self.key_imp_link
         if self.key_settings not in yamldict:
             return
         if not yamldict[self.key_settings]:
             return
         config = yamldict[self.key_settings]
-        if key not in config:
+        if old_key not in config:
             return
-        if config[key]:
+        if config[old_key]:
             config[newkey] = self.lnk_link
         else:
             config[newkey] = self.lnk_nolink
-        del config[key]
+        del config[old_key]
         self._log.warn('deprecated \"link_by_default\"')
         self._dirty = True
         self._dirty_deprecated = True
 
     def _fix_deprecated_dotfile_link(self, yamldict):
         """fix deprecated link in dotfiles"""
+        old_key = 'link_children'
         if self.key_dotfiles not in yamldict:
             return
         if not yamldict[self.key_dotfiles]:
@@ -954,14 +967,14 @@ class CfgYaml:
                 self._dirty_deprecated = True
                 self._log.warn('deprecated \"link\" value')
 
-            elif self.key_dotfile_link_children in dotfile and \
-                    type(dotfile[self.key_dotfile_link_children]) is bool:
+            elif old_key in dotfile and \
+                    type(dotfile[old_key]) is bool:
                 # patch link_children: <bool>
-                cur = dotfile[self.key_dotfile_link_children]
+                cur = dotfile[old_key]
                 new = self.lnk_nolink
                 if cur:
                     new = self.lnk_children
-                del dotfile[self.key_dotfile_link_children]
+                del dotfile[old_key]
                 dotfile[self.key_dotfile_link] = new
                 self._dirty = True
                 self._dirty_deprecated = True
@@ -998,9 +1011,38 @@ class CfgYaml:
             content = self._yaml_load(path)
         except Exception as e:
             self._log.err(e)
-            raise YamlException('invalid config: {}'.format(path))
+            raise YamlException('config format error: {}'.format(path))
+
         return content
 
+    def _validate(self, yamldict):
+        """validate entries"""
+        if not yamldict:
+            return
+
+        # check top entries
+        for e in self.top_entries:
+            if e not in yamldict:
+                err = 'no {} entry found'.format(e)
+                self._log.err(err)
+                raise YamlException('config format error: {}'.format(err))
+
+        # check link_dotfile_default
+        if self.key_settings not in yamldict:
+            # no configs top entry
+            return
+        if not yamldict[self.key_settings]:
+            # configs empty
+            return
+        settings = yamldict[self.key_settings]
+        if self.key_settings_link_dotfile_default not in settings:
+            return
+        val = settings[self.key_settings_link_dotfile_default]
+        if val not in self.allowed_link_val:
+            err = 'bad value: {}'.format(val)
+            self._log.err(err)
+            raise YamlException('config content error: {}'.format(err))
+
     def _yaml_load(self, path):
         """load from yaml"""
         with open(path, 'r') as f:
@@ -1069,12 +1111,21 @@ class CfgYaml:
             new[k] = vt
         return new
 
-    def _template_dotfiles_paths(self):
-        """template dotfiles paths"""
+    def _template_dotfiles_entries(self):
+        """template dotfiles entries"""
         if self._debug:
-            self._dbg('templating dotfiles paths')
+            self._dbg('templating dotfiles entries')
         dotfiles = self.dotfiles.copy()
 
+        # make sure no dotfiles path is None
+        for dotfile in dotfiles.values():
+            src = dotfile[self.key_dotfile_src]
+            if src is None:
+                dotfile[self.key_dotfile_src] = ''
+            dst = dotfile[self.key_dotfile_dst]
+            if dst is None:
+                dotfile[self.key_dotfile_dst] = ''
+
         # only keep dotfiles related to the selected profile
         pdfs = []
         pro = self.profiles.get(self._profile, [])
@@ -1105,6 +1156,17 @@ class CfgYaml:
             dst = dotfile[self.key_dotfile_dst]
             newdst = self.resolve_dotfile_dst(dst, templater=self._tmpl)
             dotfile[self.key_dotfile_dst] = newdst
+            # normalize the link value
+            if self.key_dotfile_link in dotfile:
+                # link
+                link = dotfile[self.key_dotfile_link]
+                newlink = self._template_item(link)
+                dotfile[self.key_dotfile_link] = newlink
+                # check link value
+                if newlink not in self.allowed_link_val:
+                    err = 'bad value: {}'.format(newlink)
+                    self._log.err(err)
+                    raise YamlException('config content error: {}'.format(err))
 
     def _rec_resolve_variables(self, variables):
         """recursive resolve variables"""
@@ -1246,7 +1308,8 @@ class CfgYaml:
         """return copy of entry from yaml dictionary"""
         if key not in dic:
             if mandatory:
-                raise YamlException('invalid config: no {} found'.format(key))
+                err = 'invalid config: no entry \"{}\" found'.format(key)
+                raise YamlException(err)
             dic[key] = {}
             return deepcopy(dic[key])
         if mandatory and not dic[key]:
@@ -1258,11 +1321,17 @@ class CfgYaml:
         """recursively delete all none/empty values in a dictionary."""
         new = {}
         for k, v in dic.items():
-            if k == self.key_dotfiles and v:
+            if k == self.key_dotfile_src:
+                # allow empty dotfile src
+                new[k] = v
+                continue
+            if k == self.key_dotfile_dst:
+                # allow empty dotfile dst
                 new[k] = v
                 continue
             newv = v
             if isinstance(v, dict):
+                # recursive travers dict
                 newv = self._clear_none(v)
                 if not newv:
                     # no empty dict
